@@ -1,16 +1,54 @@
-AP@߇*Z>W!]K@Y/@!L䭩#q n3>F ;*Ֆةs_{vZLVqj`ZB?'{!}Un
-Q⡗IApPOp(М)sNtxRKCK'<-#{0I,àWCVzZ$^mb\㝖/Q<ɨL(^r;-w_vZݟئ'RuK-׍
-BnX?,y]&;|b1b|=.fЏQGf"+JƜt]a{Jy6bWg쎁2T7ﳜ:Aj8c80S]AK*Z2O[)48ŢZAe'B Ǵ\Bw|dN4\@o}@IQNidnygZY\
-qs49Q`Ewh" GOsvEvO :ie._MxIgf>8:S\NdsZ	~FD^'Ss*{GЏ1YX\:/~l&/N˨>PeG&_$?Tuno5
-s3BeH&#LT{21ZZ:eUTNS(D¤dU$C?%|(V<OL:͖:ZWgj-7ҤV͍)|cܐ(	]g>&@&?
-hҵX9a
-11[]@0=0G_
-FSXZ!VI&=83Hسl=Wx%}5] 6LxT^ؓlj8𷈖ȮuOӷ<RzL|ʑd}u_Moem=հ=z09Fҽ(/UAbDۍ|0EWږP
-wCfU`lSK_q8[$Lbe-MNI 4zuhFp\#@3(dPGվ;>*yhl
-K.Ӱý-	dBL09fQsKPmkdթo	Lw!RY;p	ݑ(-mL"y"cj;HzvH$<30%[.зC=Demh/Cs20t'!]s]y*Ts`QX:SNd>Jx(^DȴD^%/~Ⲹ(-6D{k
-Lz7\iRУm0go'~ ]իym0I{,"pG*7[CTOVCnY-?OlXszıC
-^WSLce9HY5,6ġ9֐|ӦUp$^Zä!^YMrKLՄ^)٤5̿ał0qQx
-}K_ih.ih_key,
+/*
+ * Copyright 1996-2004 by Hans Reiser, licensing governed by 
+ * reiserfsprogs/README
+ */
+
+#include "fsck.h"
+
+static int do_items_have_the_same_type (struct item_head * ih, struct key * key)
+{
+    return (get_type (&ih->ih_key) == get_type (key)) ? 1 : 0;
+}
+
+static int are_items_in_the_same_node (struct path * path)
+{
+  return (PATH_LAST_POSITION (path) < B_NR_ITEMS (PATH_PLAST_BUFFER (path)) - 1) ? 1 : 0;
+}
+
+static void cut_last_unfm_pointer (struct path * path, struct item_head * ih)
+{
+    set_ih_free_space(ih, 0);
+    if (I_UNFM_NUM (ih) == 1)
+	reiserfsck_delete_item (path, 0);
+    else
+	reiserfsck_cut_from_item (path, -((int)UNFM_P_SIZE));
+}
+
+/*
+    if this is not a symlink - make it of_this_size;
+    otherwise find a size and return it in symlink_size;
+*/
+static unsigned long indirect_to_direct (struct path * path, __u64 len, int symlink)
+{
+    struct buffer_head * bh = PATH_PLAST_BUFFER (path);
+    struct item_head * ih = PATH_PITEM_HEAD (path);
+    __u32 unfm_ptr;
+    struct buffer_head * unfm_bh = 0;
+    struct item_head ins_ih;
+    char * buf;
+    __u32 * indirect;
+    char bad_drct[fs->fs_blocksize];
+
+    /* direct item to insert */
+    memset (&ins_ih, 0, sizeof (ins_ih));
+    if (symlink) {
+	set_ih_key_format (&ins_ih, KEY_FORMAT_1);
+    } else {
+	set_ih_key_format (&ins_ih, get_ih_key_format (ih));
+    }
+    set_key_dirid (&ins_ih.ih_key, get_key_dirid (&ih->ih_key));
+    set_key_objectid (&ins_ih.ih_key, get_key_objectid (&ih->ih_key));
+    set_type_and_offset (get_ih_key_format (&ins_ih), &ins_ih.ih_key,
 			 get_offset (&ih->ih_key) + (I_UNFM_NUM (ih) - 1) * bh->b_size, 
 			 TYPE_DIRECT);
 
@@ -187,7 +225,7 @@ int are_file_items_correct (struct item_head * sd_ih, void * sd, __u64 * size, _
 	    }
 	
 	    // does not change path
-	    next_key = get_next_key_2 (&path);
+	    next_key = reiserfs_next_key(&path);
 
 	    if (next_key == 0 || not_of_one_file (key, next_key) || 
 		(!is_indirect_key (next_key) && !is_direct_key(next_key))) 
