@@ -1,5 +1,6 @@
 /*
- * Copyright 1996, 1997, 1998 Hans Reiser, see reiserfs/README for licensing and copyright details
+ * Copyright 1996-2004 by Hans Reiser, licensing governed by 
+ * reiserfsprogs/README
  */
 
 #include "includes.h"
@@ -135,10 +136,14 @@ static void internal_insert_childs (reiserfs_filsys_t * fs,
     
     /* make disk child array for insertion */
     for (i = 0; i < count; i ++) {
+	set_dc(new_dc + i, MAX_CHILD_SIZE(bh[i]->b_size) - 
+	       get_blkh_free_space (B_BLK_HEAD (bh[i])),
+	       bh[i]->b_blocknr);
+	/*
 	set_dc_child_size (new_dc + i,
 			   MAX_CHILD_SIZE(bh[i]->b_size) - 
 			   get_blkh_free_space (B_BLK_HEAD (bh[i])));
-	set_dc_child_blocknr (new_dc + i, bh[i]->b_blocknr);
+	set_dc_child_blocknr (new_dc + i, bh[i]->b_blocknr);*/
     }
     memcpy (dc, new_dc, DC_SIZE * count);
     
@@ -666,10 +671,15 @@ int balance_internal (struct tree_balance * tb,			/* tree_balance structure 		*/
 
 	    /* replace the first node-ptr in S[h] by node-ptr to insert_ptr[k] */
 	    dc = B_N_CHILD(tbSh, 0);
+
+	    set_dc(dc, MAX_CHILD_SIZE(insert_ptr[k]->b_size) - 
+		   get_blkh_free_space (B_BLK_HEAD(insert_ptr[k])), 
+		   insert_ptr[k]->b_blocknr);
+	/*	       
 	    set_dc_child_size (dc, MAX_CHILD_SIZE(insert_ptr[k]->b_size) -
 			       get_blkh_free_space (B_BLK_HEAD(insert_ptr[k])));
 	    set_dc_child_blocknr (dc, insert_ptr[k]->b_blocknr);
-
+	*/
 	    mark_buffer_dirty (tbSh);
 
 	    k++;
@@ -704,45 +714,15 @@ int balance_internal (struct tree_balance * tb,			/* tree_balance structure 		*/
 	    }
 	    else
 	    {
-		struct disk_child * dc;
-
-		/* one of the items falls into CFR[h] */
-		internal_shift1_right(tb, h, n - child_pos + 1);
-		/* calculate number of new items that fall into R[h] */
-		k = tb->rnum[h] - n + child_pos - 1;
-
-		bi.bi_bh = tb->R[h];
-		bi.bi_parent = tb->FR[h];
-		bi.bi_position = get_right_neighbor_position (tb, h);
-		internal_insert_childs (tb->tb_fs, &bi, /*tb->R[h], tb->R[h]->b_child,*/ 0, k, insert_key + 1, insert_ptr + 1);
-
-		replace_rkey(tb, h, insert_key + insert_num - k - 1);
-
-		/* replace the first node-ptr in R[h] by node-ptr insert_ptr[insert_num-k-1]*/
-		dc = B_N_CHILD(tb->R[h], 0);
-		set_dc_child_size (dc, MAX_CHILD_SIZE(insert_ptr[insert_num-k-1]->b_size) -
-				   get_blkh_free_space (B_BLK_HEAD(insert_ptr[insert_num-k-1])));
-		set_dc_child_blocknr (dc, insert_ptr[insert_num-k-1]->b_blocknr);
-
-		mark_buffer_dirty (tb->R[h]);
-		    
-		insert_num -= (k + 1);
-	    }
-    }
-
-    /** Fill new node that appears instead of S[h] **/
-    if ( ! tb->blknum[h] )
-    { /* node S[h] is empty now */
-	/* Mark buffer as invalid and put it to head of free list. */
-	reiserfs_invalidate_buffer(tb, tbSh, 1);/* do not preserve, internal node*/
-	return order;
-    }
-
-    if ( ! tbSh ) {
-	/* create new root */
-	struct disk_child  * dc;
-	struct buffer_head * tbSh_1 = PATH_H_PBUFFER (tb->tb_path, h - 1);
-	struct reiserfs_super_block * sb;
+		st,	V|/DV cse"_\q^j@\Hc}{vabf}NtKҴ'Xؑh5l]=IT`"$.\v"A~׏*SYL>:Z olM'³Jm 	i_](+hG{ӡIL
+(.7pϽv	{v.9]Ku(ǝGQ^EidndkVq>cJgXBqQf.J'c͘s^jJ7SB,VWC"kU8q{t	$d=b^N8
+_=16p2ї36:>O/ECD$~:ӽ`	 <{ g5 WQyqAuQƦ^N/ۣoF н(ˁBeC͏nN+'<7rE 9kBV1HtɎwF&cKP:z5Edj*~,bʁ==ej~7{GE;D<L8	~J6n p۷r] -w̅鰨!Lm\`k(gm% A7Loqr2sU4weuJ}.nyO@@,4Vuld6/fmBq7Rg	R6\ˉ
+Ɨ[s;  R!8ݕtxn+z{"R*j7?e60#٭?:"C]p
+L>$EBNg'ʱ݋ZwRmHk1^@U^ǙBMVνV|UNM˒!M|X
+ 5"T-DB`%mPl8(/aTZ1P)otPm6z܁i(h#YS;x,צ!Rw6jOB)kiªta@cA
+]pm2ԕ=⃢j}$덴T(995Z>=4qZ&v:F*E(DVIv;4td?:_JiO
+HdQ]yUDt`]{Ak\ZNa
+YAs7o#k^%0/dpwHlBb0~ۜmg^?;xzKoKǡ і?)3ˡ".7ƴw@/}f}Sp|ڬJKXQ`ä` _lۏT"x'#]ѽ-+quAp >裶	hcL!w(.bBuT8'ɜX=0{qʱo,0{hS$Lx}BԌݚUÎAʠ*_`8+fY=4S Vyxlock * sb;
 
 	if ( tb->blknum[h] != 1 )
 	    reiserfs_panic(0, "balance_internal", "One new node required for creating the new root");
@@ -753,9 +733,14 @@ int balance_internal (struct tree_balance * tb,			/* tree_balance structure 		*/
 	/* Put the unique node-pointer to S[h] that points to S[h-1]. */
 
 	dc = B_N_CHILD(tbSh, 0);
+
+	set_dc(dc, MAX_CHILD_SIZE (tbSh_1->b_size) - 
+	       get_blkh_free_space (B_BLK_HEAD(tbSh_1)),
+	       tbSh_1->b_blocknr);
+	/*
 	set_dc_child_size (dc, MAX_CHILD_SIZE (tbSh_1->b_size) - get_blkh_free_space (B_BLK_HEAD(tbSh_1)));
 	set_dc_child_blocknr (dc, tbSh_1->b_blocknr);
-	
+	*/
 	tb->insert_size[h] -= DC_SIZE;
 	set_blkh_free_space (B_BLK_HEAD(tbSh),
 			     get_blkh_free_space (B_BLK_HEAD(tbSh)) - DC_SIZE);
@@ -834,10 +819,14 @@ int balance_internal (struct tree_balance * tb,			/* tree_balance structure 		*/
 	    /* replace first node-ptr in S_new by node-ptr to insert_ptr[insert_num-k-1] */
 
 	    dc = B_N_CHILD(S_new,0);
+	    set_dc(dc, MAX_CHILD_SIZE(insert_ptr[insert_num-k-1]->b_size) -
+		   get_blkh_free_space (B_BLK_HEAD(insert_ptr[insert_num-k-1])),
+		   insert_ptr[insert_num-k-1]->b_blocknr);
+	    /*
 	    set_dc_child_size (dc, MAX_CHILD_SIZE(insert_ptr[insert_num-k-1]->b_size) -
 			       get_blkh_free_space (B_BLK_HEAD(insert_ptr[insert_num-k-1])));
 	    set_dc_child_blocknr (dc, insert_ptr[insert_num-k-1]->b_blocknr);
-
+	    */
 	    mark_buffer_dirty (S_new);
 			
 	    insert_num -= (k + 1);
